var Reminder = {
    mainWorkerId: null,
    lastSelected: null,
    loadOptions: function() {
        var me = this;
        chrome.storage.sync.get(Common.options, function(items) {
            var oldOptions = Common.options;
            Common.options = items;

            if (Common.options.REMINDER) {
                if (!me.mainWorkerId) {
                    me.startMainWorker();
                } else if (oldOptions.REMINDER_INTERVAL !== Common.options.REMINDER_INTERVAL) {
                    me.stopMainWorker();
                    me.startMainWorker();
                }
            } else {
                Common.sendMessageToAllTabs({
                    action: 'closeallreminders'
                });
                me.stopMainWorker();
            }
        });
    },
    remove: function(data) {
        var me = this;
        chrome.storage.local.get(null, function(items) {
            if (items.words && items.words[data.srctext] && items.words[data.srctext][data.dstlang]) {
                delete items.words[data.srctext][data.dstlang];
                if (Object.keys(items.words[data.srctext]).length === 0) {
                    delete items.words[data.srctext];
                }
            }
            chrome.storage.local.set(items, function() {
                me.lastSelected = null;
                Common.log("removed");
            });
        });
    },
    save: function(data) {
        /**
         *
         *   srctext: sentences,
         *   dsttext: translation,
         *   dictblock : dictblock,
         *   srclang: srclang,
         *   dstlang: dstlang
         *
         **/
        data.rating = data.rating || {};
        var defaultItem = {
            words: {}
        };
        defaultItem.words[data.srctext] = {};
        defaultItem.words[data.srctext][data.dstlang] = {};
        chrome.storage.local.get(defaultItem, function(items) {
            items.words[data.srctext][data.dstlang] = {
                dsttext: data.dsttext,
                dictblock : data.dictblock,
                srclang: data.srclang,
                rating: {
                    correct: data.rating.correct || 0,
                    wrong: data.rating.wrong || 0
                }
            };
            chrome.storage.local.set(items, function() {
                Common.log("saved");
            });
        });
    },

    updateRating: function(srctext, dstlang, key) {
        this.stopMainWorker();
        this.lastSelected = null;
        this.startMainWorker();
        chrome.storage.local.get(null, function(items) {
            if (items && items.words) {
                if (items.words[srctext] && items.words[srctext][dstlang]) {
                    items.words[srctext][dstlang].rating[key] += 1;

                    //save
                    chrome.storage.local.set(items, function() {
                        Common.log('rating updated');
                    });

                    Common.sendMessageToAllTabs({
                        action: 'closeallreminders'
                    });

                }
            }
        });
    },

    markAsWrong: function(srctext, dstlang) {
        this.updateRating(srctext, dstlang, "wrong");
    },

    markAsCorrect: function(srctext, dstlang) {
        this.updateRating(srctext, dstlang, "correct");
    },

    mainWorker: function() {
        var me = this;
        if (!me.lastSelected) {
            me.checkStatus(function(status) {
                me.stopMainWorker();
                if (!status.isIdle) {
                    Common.log("worker in progress and its not idle");
                    chrome.storage.local.get(null, function(items) {
                        if (items && items.words) {
                            me.lastSelected = me.chooseByRating(items.words);
                            Common.sendMessageToCurrentTab({
                                action: "showreminder",
                                item: me.lastSelected
                            });
                        }
                    });
                    me.startMainWorker();
                } else {
                    Common.log('idle');
                    me.startMainWorker(2000);
                }
            });
        }
    },


    wordsToArray: function(words) {
        var list = [];
        for (var srctext in words) {
            for (var dstlang in words[srctext]) {
                var item = words[srctext][dstlang];
                var rating = this.score(item) || 0;
                item.dstlang = dstlang;
                item.srctext = srctext;
                item.score = rating;
                list.push(item);
            }
        }
        return list;
    },

    shortByRate: function(list) {
        return list.sort(function(a, b) {
            return a.score - b.score;
        });
    },

    chooseByRating: function(words) {
        var SLICE_LIMIT = 5;

        var list = this.wordsToArray(words);

        var sorted = this.shortByRate(list);

        var sliced = sorted.slice(0, SLICE_LIMIT);
        var rnditem = this.randomFromArray(sliced);
        return rnditem;
    },

    randomFromArray: function(items) {
        return items[Math.floor(Math.random() * items.length)];
    },

    score: function(item) {
        var rate;
        if (item.rating.correct + item.rating.wrong <= 10) {
            rate = 100 * item.rating.correct / (item.rating.correct + (10 - item.rating.correct));
        } else {
            rate = 100 * item.rating.correct / (item.rating.correct + item.rating.wrong);
        }
        return rate;
    },

    showLastSelected: function() {
        if (Common.options.REMINDER) {
            if (this.lastSelected) {
                Common.sendMessageToCurrentTab({
                    action: 'showreminder',
                    item: this.lastSelected
                });
            }
        }
    },

    startMainWorker: function(interval) {
        Common.log('Reminder Worker started at interval : ' + Common.options.REMINDER_INTERVAL);
        interval = interval || (Common.options.REMINDER_INTERVAL);
        this.mainWorkerId = setInterval(this.mainWorker.bind(this), interval);
    },
    stopMainWorker: function() {
        Common.log('Reminder worker stopped');
        clearInterval(this.mainWorkerId);
        this.mainWorkerId = null;
    },
    checkStatus: function(callback) {
        Common.sendMessageToCurrentTab({
            action: "idlecheck"
        }, function(response) {
            Common.log(JSON.stringify(response));
            if(response){
                callback({
                    isIdle: response.isIdle
                });
            } else {
                callback({
                    isIdle: true
                });
            }
        });
    }
};