var ReminderHelper = {
    IDLE_TIMEOUT: 5000,
    isIdle: false,
    container: null,
    idleEvents: ["mousemove", "keypress", "scroll"],
    CONTAINER_TEMPLATE: ['<div class="header___extension">',
        '<div class="header-btn___extension confirm-btn___extension">+</div>',
        '<div class="header-btn___extension close-btn___extension">-</div>',
        '<div class="header-btn___extension remove-btn___extension">x</div>',
        '</div>',
        '<div class="srctext___extension">',
        '',
        '</div>',
        '<div class="detail___extension">',
        '<div class="separator___extension"></div>',
        '<div class="dsttext___extension"></div>',
        '<div class="dictblock___extension"></div>',
        '<div class="detail-bottom___extension">',
        '<div class="sol___extension">',
        '<progress class="progress___extension" value="" max="100" min="0"></progress>',
        '</div>',
        '<div class="sag___extension dstlang___extension"></div>',
        '<div style="clear:both;"></div>',
        '</div>',
        '</div>',
        '<div class="expander___extension">',
        '<span class="dots___extension">&middot;&middot;&middot;</span>',
        '</div>'
    ].join("\n"),

    pingAlive: function() {
        this.isIdle = false;
    },

    getIsIdle: function() {
        return this.isIdle && Common.optionsLoaded;
    },
    getScrollWidth: function() {
        if (!this.widthWithScroll) {
            var $outer = $('<div>').css({
                    visibility: 'hidden',
                    width: 100,
                    overflow: 'scroll'
                }).appendTo('body'),
                widthWithScroll = $('<div>').css({
                    width: '100%'
                }).appendTo($outer).outerWidth();
            $outer.remove();
            this.widthWithScroll = 100 - widthWithScroll;
        }
        return this.widthWithScroll;
    },
    getScrollHeight: function() {
        if (!this.heightWithScroll) {
            var $outer = $('<div>').css({
                    visibility: 'hidden',
                    height: 100,
                    overflow: 'scroll'
                }).appendTo('body'),
                heightWithScroll = $('<div>').css({
                    height: '100%'
                }).appendTo($outer).outerHeight();
            $outer.remove();
            this.heightWithScroll = 100 - heightWithScroll;
        }
        return this.heightWithScroll;
    },
    getScrollBarState: function() {
        var result = {
            vertical: true,
            horizontal: true
        };
        try {
            var root = document.compatMode == 'BackCompat' ? document.body : document.documentElement;
            result.vertical = root.scrollHeight > root.clientHeight;
            result.horizontal = root.scrollWidth > root.clientWidth;
        } catch (e) {}
        return (result);
    },
    getWindowInnerWidth: function() {
        if (!this.getScrollBarState().vertical) {
            return window.innerWidth;
        } else {
            return window.innerWidth - this.getScrollWidth();
        }
    },
    getWindowInnerHeight: function() {
        if (!this.getScrollBarState().horizontal) {
            return window.innerHeight;
        } else {
            return window.innerHeight - this.getScrollHeight();
        }
    },
    setBindings: function() {
        var debouncedIdleChecker = Translate.debounce(function() {
            ReminderHelper.isIdle = true;
        }, this.IDLE_TIMEOUT);

        for (var i = 0; i < this.idleEvents.length; i++) {
            var event = this.idleEvents[i];
            document.addEventListener(event, this.pingAlive.bind(this));
            document.addEventListener(event, debouncedIdleChecker);
        }
    },

    getOffsetInsideWindow: function(offset) {
        if (offset.left > $(window).scrollLeft() + this.getWindowInnerWidth() - $(this.container).outerWidth()) {
            offset.left = $(window).scrollLeft() + this.getWindowInnerWidth() - $(this.container).outerWidth();
        }

        if (offset.top > $(window).scrollTop() + this.getWindowInnerHeight() - $(this.container).outerHeight()) {
            offset.top = $(window).scrollTop() + this.getWindowInnerHeight() - $(this.container).outerHeight();
        }

        if (offset.left < 0 + $(window).scrollLeft()) {
            offset.left = 0 + $(window).scrollLeft();
        }

        if (offset.top < 0 + $(window).scrollTop()) {
            offset.top = 0 + $(window).scrollTop();
        }
        return offset;
    },

    rePositionReminder: function() {
        if (this.container) {
            var offset = {
                top: $(window).scrollTop() + this.getWindowInnerHeight() - Common.options.REMINDER_LAST_BOTTOM_MARGIN,
                left: $(window).scrollLeft() + this.getWindowInnerWidth() - Common.options.REMINDER_LAST_RIGHT_MARGIN
            };
            var offsetInsideWindow = this.getOffsetInsideWindow(offset);

            $(this.container).offset({
                top: offsetInsideWindow.top,
                left: offsetInsideWindow.left
            });
        }
    },

    showReminder: function(item) {
        if (window.self === window.top) {
            var me = this;
            if (!this.container || this.container.style.display !== "block") {
                this.generateContainerDOM(item);
            }

            this.container.style.display = 'block';
            this.rePositionReminder();
            this.showAnimated(this.container);
            // alert(JSON.stringify($(this.container).offset()));
        }
    },

    showAnimated: function(el) {
        var ANIMATE_LENGTH = 100;
        var curOffset = $(el).offset();
        $(el).offset({
            top: curOffset.top - ANIMATE_LENGTH,
            left: curOffset.left
        });
        el.style.display = 'block';
        $(el).animate({
            top: '+=' + ANIMATE_LENGTH,
            left: '=' + curOffset.left
        });
    },

    closeReminder: function() {
        if (this.container) {
            this.container.style.display = 'none';
        }
    },

    saveOffset: function() {
        this.debouncedSaveOffset = this.debouncedSaveOffset || Translate.debounce(function() {
            chrome.storage.local.set({
                REMINDER_LAST_BOTTOM_MARGIN: Common.options.REMINDER_LAST_BOTTOM_MARGIN,
                REMINDER_LAST_RIGHT_MARGIN: Common.options.REMINDER_LAST_RIGHT_MARGIN,
            }, function() {
                Common.log('new offset saved');
            });
        }, 250);
        this.debouncedSaveOffset();
    },
    /*    onDragEnd: function() {
            this.saveOffset();
        },*/
    onReminderDragging: function(e, info) {
        var newinfo = this.getOffsetInsideWindow(info);
        info.top = newinfo.top;
        info.left = newinfo.left;

        Common.options.REMINDER_LAST_BOTTOM_MARGIN = $(window).scrollTop() + this.getWindowInnerHeight() - info.top;
        Common.options.REMINDER_LAST_RIGHT_MARGIN = $(window).scrollLeft() + this.getWindowInnerWidth() - info.left;
        this.saveOffset();
    },
    generateContainerDOM: function(item) {
        if (!this.container) {
            var props = {
                node: "div",
                cls: "reminder___extension",
                style: "display:none;"
            };
            this.container = Translate.createDomElement(props);

            this.container.innerHTML = this.CONTAINER_TEMPLATE;
            this.container.getElementsByClassName('expander___extension')[0].addEventListener('click', this.onExpanderClick.bind(this));
            this.container.getElementsByClassName('close-btn___extension')[0].addEventListener('click', this.onCloseButtonClick.bind(this));
            this.container.getElementsByClassName('remove-btn___extension')[0].addEventListener('click', this.onRemoveButtonClick.bind(this));
            this.container.getElementsByClassName('confirm-btn___extension')[0].addEventListener('click', this.onConfirmButtonClick.bind(this));

            $(this.container).drags();

            $(this.container).on('dragging', this.onReminderDragging.bind(this));
            // $(this.container).on('dragend', this.onDragEnd.bind(this));

            document.getElementsByTagName("body")[0].appendChild(this.container);
        }

        //hide detail
        this.container.getElementsByClassName('detail___extension')[0].style.display = 'none';

        this.container.getElementsByClassName('srctext___extension')[0].innerText = item.srctext;
        this.container.getElementsByClassName('dictblock___extension')[0].innerText = item.dictblock;
        this.container.getElementsByClassName('dsttext___extension')[0].innerText = item.dsttext;
        this.container.getElementsByClassName('dstlang___extension')[0].innerText = item.dstlang;
        this.container.getElementsByClassName('progress___extension')[0].value = item.score;


        return this.container;
    },

    onExpanderClick: function(e) {
        if (e.button === 0) {
            if (e.target.getAttribute('class') === 'dots___extension') {
                var detail = e.target.parentNode.parentNode.getElementsByClassName('detail___extension')[0];
                if (detail.style.display === 'block') {
                    detail.style.display = 'none';
                } else {
                    detail.style.display = 'block';
                }
            }
            this.rePositionReminder();
        }
    },

    getCurrentItem: function() {
        var srctext = this.container.getElementsByClassName('srctext___extension')[0].innerText;
        var dstlang = this.container.getElementsByClassName('dstlang___extension')[0].innerText;

        return {
            srctext: srctext,
            dstlang: dstlang
        };
    },

    notifyCurrentAsRemove: function() {
        var item = this.getCurrentItem();
        chrome.runtime.sendMessage({
            action: 'remove',
            srctext: item.srctext,
            dstlang: item.dstlang
        });
    },

    notifyCurrentAsWrong: function() {
        var item = this.getCurrentItem();
        chrome.runtime.sendMessage({
            action: 'markaswrong',
            srctext: item.srctext,
            dstlang: item.dstlang
        });
    },
    notifyCurrentAsCorrect: function() {
        var item = this.getCurrentItem();
        chrome.runtime.sendMessage({
            action: 'markascorrect',
            srctext: item.srctext,
            dstlang: item.dstlang
        });
    },

    onCloseButtonClick: function(e) {
        if (e.button === 0) {
            this.notifyCurrentAsWrong();
            this.closeReminder();
        }
    },

    onRemoveButtonClick: function(e) {
        if (e.button === 0) {
            this.notifyCurrentAsRemove();
            this.closeReminder();
        }
    },

    onConfirmButtonClick: function(e) {
        if (e.button === 0) {
            this.notifyCurrentAsCorrect();
            this.closeReminder();
        }
    }
};