var Popup = {
    init: function() {
        this.btnsearch = document.getElementById("search");
        this.txtquery = document.getElementById("query");
        this.container = document.getElementById("translationcontainer");
        this.loadingContainer = document.getElementById("loading");
        this.txtquery.focus();
        this.searchForSelectedText();
        this.hideContentScriptContainer();
        this.stopSpeak();
        this.setBindings();
    },
    stopSpeak: function() {
        chrome.runtime.sendMessage({
            action: "stopspeak"
        });
    },
    hideContentScriptContainer: function() {
        this.sendMessageToCurrentTab({
            action: "hidecontainer"
        });
    },
    setBindings: function() {
        this.btnsearch.addEventListener("click", this.onBtnSearchClick.bind(this));
        this.txtquery.addEventListener("keydown", this.onTxtQueryKeydown.bind(this));
        this.container.addEventListener("mousedown", this.onContainerMouseDown.bind(this));
        this.connectionPort = chrome.runtime.connect({
            name: "popup"
        });
    },
    hasClass: function(element, cls) {
        var className = " " + cls + " ";
        return (" " + element.className + " ").replace(/[\n\t]/g, " ").indexOf(className) > -1;
    },
    onBtnSpeakerMouseDown: function(el) {
        if (this.hasClass(el, "checked___extension")) {
            el.setAttribute("class", "speakerimg___extension");
            this.stopSpeak();
        } else {
            el.setAttribute("class", "speakerimg___extension checked___extension");
            var sentences = this.container.getElementsByClassName("origsentences___extension")[0].innerText;
            var srclang = this.container.getElementsByClassName("origsentences___extension")[0].getAttribute("data-srclang");
            this.speakTextMessage(srclang, sentences);
        }
    },
    onBtnAddButtonMouseDown: function(el) {
        var act;
        if (this.hasClass(el, "checked___extension")) {
            el.setAttribute("class", "addbuttonimg___extension");
            act = "remove";
        } else {
            el.setAttribute("class", "addbuttonimg___extension checked___extension");
            act = "save";
        }
        var sentences = this.container.getElementsByClassName("origsentences___extension")[0].innerText;
        var translation = this.container.getElementsByClassName("sentences___extension")[0].innerText;
        var dstlang = this.container.getElementsByClassName("sentences___extension")[0].getAttribute("data-dstlang");
        var srclang = this.container.getElementsByClassName("origsentences___extension")[0].getAttribute("data-srclang");
        chrome.runtime.sendMessage({
            action: act,
            srctext: sentences,
            dsttext: translation,
            srclang: srclang,
            dstlang: dstlang
        });
    },
    onContainerMouseDown: function(e) {
        if (e.button === 0) {
            if (this.hasClass(e.target, "speakerimg___extension")) {
                this.onBtnSpeakerMouseDown.call(this, e.target);
            }
            if (this.hasClass(e.target, "addbuttonimg___extension")) {
                this.onBtnAddButtonMouseDown.call(this, e.target);
            }
        }
    },
    speakTextMessage: function(srclang, text) {
        chrome.runtime.sendMessage({
            action: "speak",
            text: text,
            srclang: srclang
        });
    },
    onSpeakEnd: function() {
        if (this.container) {
            var btnspeaker = this.container.querySelector(".speakerimg___extension");
            if (btnspeaker) {
                btnspeaker.setAttribute("class", "speakerimg___extension");
            }
        }
    },
    onTxtQueryKeydown: function(event) {
        if (event.keyCode == 13) {
            this.processTranslation();
        }
    },
    sendMessageToCurrentTab: function(message, callback) {
        var args = Array.prototype.slice.call(arguments, 0);
        chrome.tabs.query({
            active: true,
            currentWindow: true
        }, function(tabs) {
            if (tabs.length > 0) {
                chrome.tabs.sendMessage.apply(chrome.tabs, [tabs[0].id].concat(args));
            }
        });
    },
    searchForSelectedText: function() {
        this.sendMessageToCurrentTab({
            action: "getselectedtext"
        }, this.handleResult.bind(this));
    },
    onBtnSearchClick: function() {
        this.processTranslation();
    },
    processTranslation: function() {
        var me = this;
        if (me.txtquery.value === "") {
            return;
        }
        me.loadingContainer.style.display = "block";
        me.container.style.display = "none";
        chrome.tabs.query({
            active: true,
            currentWindow: true
        }, function(tabs) {
            chrome.tabs.sendMessage(tabs[0].id, {
                action: "translate",
                text: me.txtquery.value
            }, me.handleResult.bind(me));
        });
    },
    handleResult: function(response) {
        if(response){
            this.container.innerHTML = response.html;
            this.txtquery.value = response.selectedText;
            this.loadingContainer.style.display = "none";
            this.container.style.display = "block";
        }
    }
};
document.addEventListener("DOMContentLoaded", Popup.init.bind(Popup));
chrome.runtime.onMessage.addListener(function(message, sender, sendResponse) {
    if (message.action === "speakend") {
        Popup.onSpeakEnd();
    }
    return true;
});